#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneController_test.py

 @author East Japan Accounting Center Co., Ltd.

 (c)2017 The University of Aizu
 This software is released under the New BSD License.

"""
import unittest

import RTC
import OpenRTM_aist
import DroneController
#import drone_distance_cal

import sys


from mock import Mock
from mock import patch, MagicMock

class TestDroneController(unittest.TestCase):
    def setUp(self):
        self.m = DroneController.DroneController(0)

        self.xx_logger = Mock()
        DroneController.logger.warning = self.xx_logger.warning
        DroneController.logger.error = self.xx_logger.error
        DroneController.logger.info = self.xx_logger.info
        DroneController.logger.debug = self.xx_logger.debug

    def test_read_drone_ctrl(self):
        xx = Mock()
        self.m._inDrCtrlIn.isNew = xx.isNew
        self.m._inDrCtrlIn.read = xx.read

        init_drone_ctrl = RTC.DroneCtrl(RTC.Time(0,0),
                                        0,
                                        RTC.TimedString(RTC.Time(0, 0), ""),
                                        RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                    100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                    100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                    0.0, 0, RTC.GPS_FIX_NORMAL),
                                        RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                        RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                        RTC.TimedString(RTC.Time(0, 0), ""))
        drone_ctrl = RTC.DroneCtrl(RTC.Time(0,0),
                                   100,
                                   RTC.TimedString(RTC.Time(0, 0), ""),
                                   RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                               37.0, 139.0, 215.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                               37.5, 139.5, 10.0, 0.0, 0.0, 0.0, 0.0,
                                               0.0, 0, RTC.GPS_FIX_NORMAL),
                                   RTC.TimedFloatSeq(RTC.Time(0, 0), [3.0, 2.0, 1.0]),
                                   RTC.TimedFloat(RTC.Time(0, 0), 45),
                                   RTC.TimedString(RTC.Time(0, 0), "Hovering"))

        self.m._d_inDrCtrl = init_drone_ctrl
        xx.read.return_value = drone_ctrl

        in_is_new = [False,True]
        id_result = [0, 100]
        home_lat_result = [100.0, 37.0]
        home_lon_result = [200.0, 139.0]
        home_alt_result = [0.0, 215.0]
        drone_lat_result = [100.0, 37.5]
        drone_lon_result = [200.0, 139.5]
        drone_alt_result = [0.0, 10.0]
        accel_x_result = [0.0, 3.0]
        accel_y_result = [0.0, 2.0]
        accel_z_result = [0.0, 1.0]
        mag_result = [0, 45]
        state_result = ["", "Hovering"]

        for i,d in enumerate(in_is_new):
            xx.isNew.return_value = in_is_new[i]
            self.m._read_drone_ctrl()

            self.assertEqual(self.m._d_inDrCtrl.id, id_result[i])            
            self.assertEqual(self.m._d_inDrCtrl.homeGPS.latitude, home_lat_result[i])
            self.assertEqual(self.m._d_inDrCtrl.homeGPS.longitude, home_lon_result[i])
            self.assertEqual(self.m._d_inDrCtrl.homeGPS.altitude, home_alt_result[i])
            self.assertEqual(self.m._d_inDrCtrl.droneGPS.latitude, drone_lat_result[i])
            self.assertEqual(self.m._d_inDrCtrl.droneGPS.longitude, drone_lon_result[i])
            self.assertEqual(self.m._d_inDrCtrl.droneGPS.altitude, drone_alt_result[i])
            self.assertEqual(self.m._d_inDrCtrl.accel.data[0], accel_x_result[i])
            self.assertEqual(self.m._d_inDrCtrl.accel.data[1], accel_y_result[i])
            self.assertEqual(self.m._d_inDrCtrl.accel.data[2], accel_z_result[i])
            self.assertEqual(self.m._d_inDrCtrl.mag.data, mag_result[i])
            self.assertEqual(self.m._d_inDrCtrl.state.data, state_result[i])

    def test_set_target_gps(self):
        xx = Mock()

        self.m._move_mode = xx._move_mode
        self.m._takeoff_landing = xx._takeoff_landing
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down
        self.m._move_continue = xx._move_continue
        #self.m._turn_dir = xx._turn_dir




        in_takeoff_landing = [1, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                              0,  0, 0, 0, 0]

        in_home_lat = [37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 
                       37.500, 37.500, 37.500, 37.500, 37.500]
        in_home_lon = [139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900,
                       139.900, 139.900, 139.900, 139.900, 139.900]
        in_home_alt = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                       0.0, 0.0, 0.0, 0.0, 0.0]

        in_front_back = [0, 0, 0, 1, 1, -1, -1, 0, 0, 0, 0, 0,
                         0, 1, 1, -1, -1]
        in_right_left = [0, 0, 0, 0, 0, -1, -1, -1, -1, -1, -1, 1,
                        1, 0, 0, -1, -1]
        in_up_down = [0, 0, 0, 0, 0, 0, 0, 1, 1, -1, -1, 0,
                      0, 0, 0, 0, 0]

        in_move_continue = [False, False, False, False, False, False, False, False, False, False, False, True, True, True, True, True, True]

        in_drone_lat = [37.500, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520,
                        37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lon = [139.900, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930,
                        139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_alt = [0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
                        8.0, 8.0, 8.0, 8.0, 8.0]

        in_target_lat = [37.500, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.530,
                        37.530, 37.530, 37.530, 37.5201, 37.5201]
        in_target_lon = [139.900, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.940,
                        139.940, 139.940, 139.940, 139.9301, 139.9301]
        in_target_alt = [0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
                        8.0, 8.0, 8.0, 8.0, 8.0]

        in_move_mode = ["Absolute", "Absolute", "Absolute", "Absolute", "Relative",
                        "Absolute", "Relative", "Absolute", "Relative", "Absolute", "Relative", "Absolute",
                        "Relative", "Absolute", "Relative", "Absolute", "Relative"]
        in_mode = [1, 1, 1, 1, 2,
                   1, 2, 1, 2, 1, 2, 1,
                   2, 1, 2, 1, 2]

        in_last_speed = [3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0,
                         3.0, 4.0, 4.0, 3.0, 3.0]

        
        in_last_front_back = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                              0, 1, 1, -1, -1]
        in_last_right_left = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
                              1, 0, 0, -1, -1]
        in_last_up_down = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0]

        
        target_lat_result = [37.500, 37.500, 37.520, 37.52041, 37.52019, 37.51959, 37.520, 37.520, 37.52019, 37.520, 37.52019, 37.530,
                           37.530, 37.52041, 37.52019, 37.51959, 37.520]

        target_lon_result = [139.900, 139.900, 139.930, 139.930, 139.93019, 139.92968, 139.92973, 139.92968, 139.92981, 139.92968, 139.92981, 139.940,
                           139.940, 139.930, 139.93019, 139.92968, 139.92973]

        target_alt_result = [10.0, 0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 9.0, 9.0, 7.0, 7.0, 8.0,
                                8.0, 8.0, 8.0, 8.0, 8.0]

        #self.m._home_gps = True
        self.m._drone_gps = True
        self.m._SpeedParam = 3.0
        self.m._d_inDrCtrl.mag.data = 45
        for test_case in range(len(target_lat_result)):
            #mode
            xx._move_mode.return_value = in_mode[test_case]
            self.m._set_move_mode()

            #gps
            xx._takeoff_landing.return_value = in_takeoff_landing[test_case]
            xx._front_back.return_value =in_front_back[test_case]
            xx._right_left.return_value =in_right_left[test_case]
            xx._up_down.return_value =in_up_down[test_case]
            xx._move_continue.return_value = in_move_continue[test_case]

            self.m._last_front_back = in_last_front_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]
            self.m._last_up_down = in_last_up_down[test_case]

            self.m._d_inDrCtrl.homeGPS.latitude = in_home_lat[test_case]
            self.m._d_inDrCtrl.homeGPS.longitude = in_home_lon[test_case]
            self.m._d_inDrCtrl.homeGPS.altitude = in_home_alt[test_case]

            self.m._d_inDrCtrl.droneGPS.latitude = in_drone_lat[test_case]
            self.m._d_inDrCtrl.droneGPS.longitude = in_drone_lon[test_case]
            self.m._d_inDrCtrl.droneGPS.altitude = in_drone_alt[test_case]

            self.m._d_outTarget.pos.latitude = in_target_lat[test_case]
            self.m._d_outTarget.pos.longitude = in_target_lon[test_case]
            self.m._d_outTarget.pos.altitude = in_target_alt[test_case]

            self.m._d_outMoveMode.data = in_move_mode[test_case]
            self.m._last_speedparam = in_last_speed[test_case]

            self.m._set_target_gps()


            self.assertEqual(round(self.m._d_outTarget.pos.latitude,5), target_lat_result[test_case])
            self.assertEqual(round(self.m._d_outTarget.pos.longitude,5), target_lon_result[test_case])
            self.assertEqual(self.m._d_outTarget.pos.altitude, target_alt_result[test_case])


    def test_set_target_base_alt(self):

        xx = Mock()

        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down

        in_last_front_back = [0, 0, 0, 0, 0, 1, 0, 0, 0, 1]
        in_last_right_left = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        in_last_up_down = [1, 0, 0, 0, -1, 1, 0, 0, 0, -1]

        in_front_back = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        in_right_left = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
        in_up_down = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        in_alt_chg_req_cancel = [False, False, True, True, True, True, False, True, True, True]
        in_alt_chg_cancel_wait_tm = [0, 0, 48, 49, 0, 0, 0, 48, 49, 0]

        in_DrCtrl_pos_alt = [10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5, 10.5]
        in_target_pos_alt = [10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, 10.0]

        out_alt_chg_req_cancel = [True, False, True, False, True, True, False, True, False, True]
        out_alt_chg_cancel_wait_tm = [0, 0, 49, 0, 0, 0, 0, 49, 0, 0]
        out_target_pos_alt = [10.5, 10.0, 10.5, 10.0, 10.5, 10.5, 10.0, 10.5, 10.0, 10.5]

        for test_case in range(len(in_last_front_back)):
            xx._front_back.return_value = in_front_back[test_case]
            xx._right_left.return_value = in_right_left[test_case]
            xx._up_down.return_value = in_up_down[test_case]
            mv = self.m._moving()

            self.m._last_front_back = in_last_front_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]
            self.m._last_up_down = in_last_up_down[test_case]

            self.m._alt_chg_req_cancel = in_alt_chg_req_cancel[test_case]
            self.m._alt_chg_cancel_wait_tm = in_alt_chg_cancel_wait_tm[test_case]

            self.m._d_inDrCtrl.droneGPS.altitude = in_DrCtrl_pos_alt[test_case]
            self.m._d_outTarget.pos.altitude = in_target_pos_alt[test_case]

            self.m._set_target_base_alt(mv)

            self.assertEqual(self.m._alt_chg_req_cancel, out_alt_chg_req_cancel[test_case])
            self.assertEqual(self.m._alt_chg_cancel_wait_tm, out_alt_chg_cancel_wait_tm[test_case])
            self.assertEqual(self.m._d_inDrCtrl.droneGPS.altitude, out_target_pos_alt[test_case])

    def test_set_target_speed(self):
        xx = Mock()

        init_speed = RTC.TimedFloat(RTC.Time(0, 0), 0.0)


        self.m._d_outTargetSpeed = init_speed


        speed = [5.0]
        result = [5.0]

        self.m._SpeedParam = speed[0]
        self.m._set_target_speed()
        self.assertEqual(self.m._d_outTarget.speed.data, result[0])
        self.assertEqual(self.m._d_outTargetSpeed.data, result[0])



    def test_set_target_yaw(self):
        xx = Mock()

        self.m._turn_dir = xx._turn_dir
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down
        self.m._move_continue = xx._move_continue
        self.m._diff_target_angle = xx._diff_target_angle

        init_yaw = RTC.TimedShortSeq(RTC.Time(0, 0), [])

        self.m._d_outTargetYaw = init_yaw

        in_turn_dir = [0, 0, -1, 1, -1, 1, 0, 0, 0, 0,
                      0, 0, 0, 0, 0]
        in_front_back = [0, 0, 0, 0, -1, 0, -1, -1, 1, 0,
                         0, -1, 1, 0, 0]
        in_right_left = [0, 0, 0, 0, 0, 1, 0, 0, 0, -1,
                         1, 0, 0, -1, 1]
        in_up_down = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0]

        in_move_continue = [False, False, False, False, False, False, False, False, False, False,
                            False, False, False, True, True]
        in_relative_mag = [45, 45, 45, 45, 45, 45, 45, 45, 45, 45,
                           45, 45, 45, 45, 45]
        in_mag = [45, 45, 45, 45, 45, 45, 45, 45, 45, 45,
                  45, 50, 50, 50, 50]
        in_move_mode = ["Absolute", "Relative", "Absolute", "Relative",
                        "Relative", "Relative", "Absolute", "Absolute", "Absolute", "Absolute",
                        "Absolute", "Relative", "Relative", "Relative", "Relative"]
        in_last_front_back = [0, 0, 0, 0, 0, 0, 0, -1, -1, 0, 1, -1, 0, 1, 0]
        in_last_right_left = [0, 0, 0, 0, 0, 0, 0, 0, 0, 1, -1, 0, 1, -1, 0]
        in_diff_target_angle = [0,0,0,0,0,0, 135, 135, -45, -135,
                                45,0,0,0,0]


        moveturnspeed = [30]
        turnspeed = [45]
        self.m._MoveTurnSpeed[0] = moveturnspeed[0]
        self.m._TurnSpeed[0] = turnspeed[0]

        result = [[0, 0], [0, 0], [-1, 0], [1,  90],
                  [-1, 15.0], [1, 75], [0, 0], [1, 180], [-1, 0], [-1, 270],
                  [ 1,   90], [ 1,   50], [ 1,  50], [ 1,  45], [ 1, 45]
                 ]

        test_case = 0
        while test_case < len(in_mag):
            xx._turn_dir.return_value = in_turn_dir[test_case]
            xx._front_back.return_value = in_front_back[test_case]
            xx._right_left.return_value = in_right_left[test_case]
            xx._up_down.return_value = in_up_down[test_case]
            xx._move_continue.return_value = in_move_continue[test_case]
            xx._diff_target_angle.return_value = in_diff_target_angle[test_case]

            self.m._relative_mag = in_relative_mag[test_case]
            self.m._d_inDrCtrl.mag.data = in_mag[test_case]
            self.m._d_outMoveMode.data = in_move_mode[test_case]
            self.m._last_front_back = in_last_front_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]

            self.m._set_target_yaw()

            self.assertEqual(self.m._d_outTarget.yaw.data, result[test_case])

            test_case += 1

    def test_set_move_mode(self):
        xx = Mock()

        self.m._move_mode = xx._move_mode

        init_movemode = RTC.TimedString(RTC.Time(0, 0), "")

        self.m._d_outMoveMode = init_movemode

        in_move = [1, 2]
        result = ["Absolute", "Relative"]

        for i, d in enumerate(in_move):
            xx._move_mode.return_value = in_move[i]
            self.m._set_move_mode()
            self.assertEqual(self.m._d_outMoveMode.data, result[i])

    def test_diff_target_angle(self):
        in_mag = [30, 30, 280, 280]
        in_target_lat = [37.520, 37.580, 37.520, 37.580]
        in_target_lon = [139.910, 139.810, 139.910, 139.810]
        in_pos_lat = [37.550, 37.500, 37.550, 37.500]
        in_pos_lon = [139.940, 139.890, 139.940, 139.890]

        deg_result = [-165.0, -75.0, -55.0, 35.0]

        test_case = 0
        while test_case < len(deg_result):
            self.m._d_inDrCtrl.mag.data = in_mag[test_case]

            self.m._d_outTarget.pos.latitude = in_target_lat[test_case]
            self.m._d_outTarget.pos.longitude = in_target_lon[test_case]
            self.m._d_inDrCtrl.droneGPS.latitude = in_pos_lat[test_case]
            self.m._d_inDrCtrl.droneGPS.longitude = in_pos_lon[test_case]

            deg = round(self.m._diff_target_angle(), 1)

            self.assertEqual(deg, deg_result[test_case])

            test_case += 1

    def test_move_mode(self):
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home

        in_home = [True, True, True, True, False, False, False, False, False, False, False, False]
        in_target_mode = [1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2]
        in_button = [0, 0, DroneController.BUTTON_10_MASK, DroneController.BUTTON_10_MASK,
                     0, 0, DroneController.BUTTON_10_MASK, DroneController.BUTTON_10_MASK,
                     0, 0, DroneController.BUTTON_10_MASK, DroneController.BUTTON_10_MASK]

        result = [1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1]

        test_case = 0
        while test_case < len(result):
            xx._check_drone_on_home.return_value = in_home[test_case]
            self.m._target_gps._mode = in_target_mode[test_case]
            self.m._d_ButtonIn.data = in_button[test_case]

            mode = self.m._move_mode()

            self.assertEqual(mode, result[test_case])

            test_case += 1

    def test_front_back(self):
        in_data = [[], [-2,0], [-1,0], [0,0], [1,0], [2,0]]
        result =  [ 0,      1,      1,     0,    -1,   -1]

        for i, d in enumerate(in_data):
            self.m._d_AnalogIn.data = in_data[i]

            self.assertEqual(self.m._front_back(), result[i])

    def test_move_continue(self):
        xx = Mock()

        self.m._turn_dir = xx._turn_dir        

        in_last_move_mode = ["Absolute", "Relative", "Absolute", "Absolute", "Absolute",
                             "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute"]
        in_move_mode = ["Relative", "Absolute", "Absolute", "Absolute", "Absolute",
                        "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute"]

        in_turn_dir = [0, 0, -1, 1, 0, 0, 0, 0, 0, 0, 0]

        in_last_flont_back = [0, 1, 0, 0, 0, 0, -1, 0, 0, 1, -1]
        in_flont_back = [0, 1, 0, 0, 0, 0, 0, 1, 0, 1, -1]

        in_last_right_left = [0, 0, 0, -1, 0, 0, 0, 1, -1, 1, 1]
        in_right_left = [0, 0, 0, -1, 0, 0, 1, 1, -1, 1, 1]

        in_last_up_down = [0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 1]
        in_up_down = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]



        result = [False, False, False, False, False, False, False, False, True, True, True]


        test_case = 0
        while test_case < len(result):
            xx._turn_dir.return_value = in_turn_dir[test_case]

            self.m._last_move_mode = in_last_move_mode[test_case]
            self.m._d_outMoveMode.data = in_move_mode[test_case]
            self.m._last_front_back = in_last_flont_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]
            self.m._last_up_down = in_last_up_down[test_case]

            moved_countinue = self.m._move_continue(in_flont_back[test_case], in_right_left[test_case], in_up_down[test_case])

            self.assertEqual(moved_countinue, result[test_case])

            test_case += 1

    def test_turn_dir(self):  
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down


        in_home = [False, True]
        in_data = [[0,-1], [0,0], [0,1]]
        in_move = [-1, 0, 1]

        result =  [[[[[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, -1, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[ 0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0,  1, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]],


                   [[[[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]]
                    ]

        self.m._d_outMoveMode.data = "Absolute"
        for i1, d1 in enumerate(in_home):
            xx._check_drone_on_home.return_value = in_home[i1]
            for i2, d2 in enumerate(in_data):
                self.m._d_AnalogIn.data = in_data[i2]
                for i3, d3 in enumerate(in_move):
                    #front_back
                    xx._front_back.return_value =in_move[i3]
                    for i4, d4 in enumerate(in_move):
                        #right_left
                        xx._right_left.return_value = in_move[i4]
                        for i5, d5 in enumerate(in_move):
                            #up_down
                            xx._up_down.return_value = in_move[i5]

                            operation = self.m._turn_dir()

                            self.assertEqual(operation, result[i1][i2][i3][i4][i5])

    def test_move_speed(self):
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home

        in_home = [False, True] 
        in_button = [0,
                     DroneController.BUTTON_05_MASK,
                     DroneController.BUTTON_07_MASK,
                     DroneController.BUTTON_05_MASK | DroneController.BUTTON_07_MASK
                     ]

        in_speed = [1.0, 2.0, 5.0,  9.0, 10.0]
        result1 = [[[1.0, 2.0, 5.0,  9.0, 10.0],
                   [2.0, 3.0, 6.0, 10.0, 10.0],
                   [1.0, 1.0, 4.0,  8.0,  9.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0]],

                   [[1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0]]
                   ]
        result2 = [[[0, 0, 0, 0, 0],
                   [0, 0, 0, 0, 0],
                   [0, 0, 0, 0, 0],
                   [1, 1, 1, 1, 1]],

                   [[0, 0, 0, 0, 0],
                   [1, 1, 1, 1, 1],
                   [1, 1, 1, 1, 1],
                   [1, 1, 1, 1, 1]]
                   ]

        for i1, d1 in enumerate(in_home):
            xx._check_drone_on_home.return_value = in_home[i1]        
            for i2, d2 in enumerate(in_button):
                self.m._d_ButtonIn.data = in_button[i2]
                for i3, d3 in enumerate(in_speed):
                    self.xx_logger.warning.call_count = 0
                    self.m._SpeedParam = in_speed[i3]

                    self.m._move_speed()

                    self.assertEqual(self.m._SpeedParam, result1[i1][i2][i3])
                    self.assertEqual(self.xx_logger.warning.call_count, result2[i1][i2][i3])

    def test_front_back_check(self):
        in_status = [-2, -1, 0, 1, 2]
        result = [-1, -1, 0, 1, 1]
        
        for i, d in enumerate(in_status):
            status = self.m._front_back_check(in_status[i])
            self.assertEqual(status, result[i])

    def test_turn_dir_check(self):
        in_dir = [-2, -1, 0, 1, 2]
        result = [-1, -1, 0, 1, 1]
        
        for i, d in enumerate(in_dir):
            dir = self.m._turn_dir_check(in_dir[i])
            self.assertEqual(dir, result[i])
            
    def test_move_speed_check(self):
        in_speed = [0.0, 1.0, 2.0, 5.0, 9.0, 10.0, 11.0]
        result  =  [1.0, 1.0, 2.0, 5.0, 9.0, 10.0, 10.0]
        
        for i, d in enumerate(in_speed):
            speed = self.m._move_speed_check(in_speed[i])
            self.assertEqual(speed, result[i])

    def test_ready_to_log_write(self):
        
        self.m._d_inDroneCtrl = RTC.DroneCtrl(RTC.Time(1513041940, 0),
                                             0,
                                             RTC.TimedString(RTC.Time(0, 0), ""),
                                             RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                         0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                         0.0, 0, RTC.GPS_FIX_NORMAL),
                                             RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                             RTC.TimedFloat(RTC.Time(0, 0), 0),
                                             RTC.TimedString(RTC.Time(0, 0), "Armed"))
        
        in_file_md = [DroneController.FILE_MODE_WRITE, DroneController.FILE_MODE_ADD_WRITE]
        self.m._file_name = ""
        self.m._file = None
        self.m._writer = None
        self.m._act_start_date, self.m._act_start_time = self.m._get_datetime_to_str()
        self.m._log_row = ["NULL"] * len(self.m._header)

        for cnt in range(len(in_file_md)):
            self.m._ready_to_log_write(in_file_md[cnt])
            self.m._close_rtc_viewer_log()
            self.assertEqual(self.m._file, None)
            self.assertEqual(self.m._writer, None)

    def test_is_open_rtc_viewer_log(self):

        in_file_md = [DroneController.FILE_MODE_WRITE, DroneController.FILE_MODE_WRITE, DroneController.FILE_MODE_WRITE, DroneController.FILE_MODE_ADD_WRITE, DroneController.FILE_MODE_ADD_WRITE, DroneController.FILE_MODE_ADD_WRITE]
        in_file_name = ["", "20171211_144623_ctrl_log", "20171211_144623_ctrl_log.csv", "", "20171212_104554_ctrl_log", "20171212_104554_ctrl_log.csv"]
        out_rtn = [False, True, True, False, True, True]

        for cnt in range(len(in_file_md)):
            self.m._file_name = in_file_name[cnt]
            rtn = self.m._is_open_rtc_viewer_log(in_file_md[cnt])
            self.assertEqual(rtn, out_rtn[cnt])

    def test_is_csv_writer(self):

        in_file = [None] 
        in_writer = [None] 
        out_rtn = [False]

        for cnt in range(len(in_file)):
            self.m._file = in_file[cnt]
            self.m._writer = in_writer[cnt]
            rtn = self.m._is_csv_writer()

    def test_conv_rtctime_to_datetime(self):
        
        tm = RTC.Time(1512971183, 993842000)
        date, time = self.m._conv_rtctime_to_datetime(tm)
        self.assertEqual(date, "20171211")
        self.assertEqual(time, "14:46:23.993")

    def test_set_rcv_log(self):

        init_ctrl = RTC.DroneCtrl(RTC.Time(0, 0),
                                0,
                                RTC.TimedString(RTC.Time(0, 0), ""),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            100.0, 200.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.0, 0.0]),
                                RTC.TimedFloat(RTC.Time(0, 0), 0.0),
                                RTC.TimedString(RTC.Time(0, 0), ""))

        ctrl1 = RTC.DroneCtrl(RTC.Time(1512971183, 993842000),
                                1,
                                RTC.TimedString(RTC.Time(0, 0), "drone1"),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.5223693848, 139.9389993, 208.09, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9387618, 9.99, 0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloatSeq(RTC.Time(0, 0), [0.0, 0.19, 0.0]),
                                RTC.TimedFloat(RTC.Time(0, 0), 89.0),
                                RTC.TimedString(RTC.Time(0, 0), "In_Move"))

        in_rtc_state = ["Inactive", "Error", "Exit", "Active", "Active"]
        in_ctrl = [init_ctrl, init_ctrl, init_ctrl, init_ctrl, ctrl1]

        out_snddate = ["NULL", "NULL", "NULL", "NULL", "20171211"]
        out_sndtime = ["NULL", "NULL", "NULL", "NULL", "14:46:23.993"]
        out_robotid = ["NULL", "NULL", "NULL", "NULL", "drone1"]
        out_id = ["NULL", "NULL", "NULL", "NULL", 1]
        out_state = ["NULL", "NULL", "NULL", "NULL", "In_Move"]
        out_h_lat = ["NULL", "NULL", "NULL", "NULL", "37.5223693848"]
        out_h_lon = ["NULL", "NULL", "NULL", "NULL", "139.9389993000"]
        out_h_alt = ["NULL", "NULL", "NULL", "NULL", "208.09"]
        out_dr_lat = ["NULL", "NULL", "NULL", "NULL", "37.5223680000"]
        out_dr_lon = ["NULL", "NULL", "NULL", "NULL", "139.9387618000"]
        out_dr_alt = ["NULL", "NULL", "NULL", "NULL", "9.99"]
        out_accx = ["NULL", "NULL", "NULL", "NULL", "0.00"]
        out_accy = ["NULL", "NULL", "NULL", "NULL", "0.19"]
        out_accz = ["NULL", "NULL", "NULL", "NULL", "0.00"]
        out_mag = ["NULL", "NULL", "NULL", "NULL", 89]

        for cnt in range(len(in_rtc_state)):
            self.m._d_inDrCtrl = in_ctrl[cnt]
            self.m._log_row = ["NULL"] * len(self.m._header)

            self.m._set_rcv_log(in_rtc_state[cnt])

            self.assertEqual(self.m._log_row[self.m._header.index("RcvDate")], out_snddate[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RcvTime")], out_sndtime[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RcvRobotID")], out_robotid[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RcvID")], out_id[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("State")], out_state[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("HomeLat")], out_h_lat[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("HomeLon")], out_h_lon[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("HomeAlt")], out_h_alt[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RobotLat")], out_dr_lat[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RobotLon")], out_dr_lon[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("RobotAlt")], out_dr_alt[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("AccX")], out_accx[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("AccY")], out_accy[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("AccZ")], out_accz[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("Mag")], out_mag[cnt])

    def test_set_snd_log(self):
    
        init_pos = RTC.DroneTarget(RTC.Time(1512971183, 993842000),
                                    0,
                                    RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                                0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                                0.0, 0, RTC.GPS_FIX_NORMAL),
                                    RTC.TimedFloat(RTC.Time(0, 0), 0),
                                    RTC.TimedShortSeq(RTC.Time(0, 0), [0, 0]))

        pos1 = RTC.DroneTarget(RTC.Time(1512971183, 993842000),
                                1,
                                RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                            37.522368, 139.9389993, 10.0, 
                                            0.0, 0.0, 0.0, 0.0,
                                            0.0, 0, RTC.GPS_FIX_NORMAL),
                                RTC.TimedFloat(RTC.Time(0, 0), 3.0),
                                RTC.TimedShortSeq(RTC.Time(0, 0), [1, 90]))

        mv_mode = RTC.TimedString(RTC.Time(1512971183, 993842000), "Absolute")
        self.m._d_outMoveMode = mv_mode
        self.m._d_inDrCtrl.robotID.data = "drone1"

        in_rtc_state = ["Inactive", "Error", "Exit", "Active", "Active"]
        in_pos = [init_pos, init_pos, init_pos, init_pos, pos1]
        in_wrote = [False, False, False, False, True]
        in_mv_md = ["NULL", "NULL", "NULL", "Absolute", "Absolute"]

        out_rcvdate = ["NULL", "NULL", "NULL", "20171211", "20171211"]
        out_rcvtime = ["NULL", "NULL", "NULL", "14:46:23.993", "14:46:23.993"]
        out_robotid = ["NULL", "NULL", "NULL", "NULL", "drone1"]
        out_id = ["NULL", "NULL", "NULL", "NULL", 1]
        out_lat = ["NULL", "NULL", "NULL", "NULL", "37.5223680000"]
        out_lon = ["NULL", "NULL", "NULL", "NULL", "139.9389993000"]
        out_alt = ["NULL", "NULL", "NULL", "NULL", "10.00"]
        out_spd = ["NULL", "NULL", "NULL", "NULL", "3.00"]
        out_yaw_dir = ["NULL", "NULL", "NULL", "NULL", 1]
        out_yaw_deg = ["NULL", "NULL", "NULL", "NULL", 90]

        for cnt in range(len(in_rtc_state)):
            self.m._d_outTarget = in_pos[cnt]
            self.m._out_target_wrote = in_wrote[cnt]
            self.m._d_outMoveMode.data = in_mv_md[cnt]
            self.m._log_row = ["NULL"] * len(self.m._header)

            self.m._set_snd_log(in_rtc_state[cnt])

            self.assertEqual(self.m._log_row[self.m._header.index("SndDate")], out_rcvdate[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("SndTime")], out_rcvtime[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("SndRobotID")], out_robotid[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("SndID")], out_id[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetLat")], out_lat[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetLon")], out_lon[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetAlt")], out_alt[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetSpd")], out_spd[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetYawDir")], out_yaw_dir[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("TargetYawDeg")], out_yaw_deg[cnt])
            self.assertEqual(self.m._log_row[self.m._header.index("MoveMode")], in_mv_md[cnt])

    def test_set_other_log(self):
    
        in_rtc_state = ["Inactive", "Active", "Error", "Exit"]

        for cnt in range(len(in_rtc_state)):
            self.m._act_start_date, self.m._act_start_time = "20171211", "14:46:23.993"
            self.m._log_row = ["NULL"] * len(self.m._header)

            self.m._set_other_log(in_rtc_state[cnt])

            self.assertEqual(self.m._log_row[self.m._header.index("Category")], "GCS")
            self.assertEqual(self.m._log_row[self.m._header.index("ActStartDate")], "20171211")
            self.assertEqual(self.m._log_row[self.m._header.index("ActStartTime")], "14:46:23.993")
            self.assertEqual(self.m._log_row[self.m._header.index("RTCState")], in_rtc_state[cnt])

                        
if __name__ == '__main__':
    unittest.main()
